;;;-----------------------
;;;-----------------------
;;; Globals
;;;-----------------------
;;;-----------------------
extensions [ palette matrix stats]

globals
[
  old-generation
  old-generation-list
  cleared?
  any-centroids-moved?
  maximumDelay
  averageradius
]

breed [individuals individual]
breed [centroids centroid]
breed [movingcents movingcent]


;coopfitness and fitness
; nn nearest neighbor based on posX
; nb nearest best neighor based on posX
; chosendistance is the distance selected in the variable whichdistance
individuals-own [coopfitness fitness posX tickdelay currentdelay euclidean chosendistance nn nb nselections]

centroids-own [closestcentroid]

movingcents-own [radius]

patches-own [
  height fitnessP
]


;;;-----------------------
;;;-----------------------
;;; mean-shift clustering after K-means
;;; radius is set to half the distance to the nearest centroid
;;;-----------------------
;;;-----------------------

to reset-movingcents
  ask centroids [
    set closestcentroid distance min-one-of centroids with [self != myself] [distance myself]
  ]

  let avgradius mean [closestcentroid] of centroids
  ask centroids [

    hatch-movingcents 1 [
      set radius avgradius
      set color black
      set pen-mode "down"
      ;set radius radius / 2.0
    ]
  ]
  ask centroids [die]
end

to recalculate-radius
    ask movingcents [
    ;let closestcentroid distance min-one-of movingcents with [self != myself] [distance myself]
    ;set radius closestcentroid * 0.75
  ]
end

to move-movingcents
  ask movingcents [
    let a radius
    let newcolor one-of base-colors

    let newxcor mean [xcor] of individuals in-radius a
    let newycor mean [ycor] of individuals in-radius a

    set xcor newxcor
    set ycor newycor

    ask individuals in-radius a [
      set color newcolor
    ]
  ]
end

to kill-movingcents-in-radius
  ask movingcents[
    let r radius  * 0.5
    ask movingcents with [self != myself] in-radius r [die]
  ]
end


;;;-----------------------
;;;-----------------------
;;; K-means
;;;-----------------------
;;;-----------------------


to reset-centroids
  set any-centroids-moved? true
  ask individuals [ set color grey ]

  let colors 0
  ask centroids [die]
  create-centroids num-centroids [
    move-to one-of individuals
    set size 7
    set color colors
    set colors colors + 1
  ]
  clear-all-plots
  reset-ticks
end

to k-means
  if not any-centroids-moved? [stop]
  set any-centroids-moved? false
  assign-clusters
  update-clusters
  tick
end

to assign-clusters
  ask individuals [set color [color] of closest-centroid]
end

to update-clusters
  let movement-threshold 0.1
  ask centroids [
    let my-points individuals with [color = [color] of myself]
    if any? my-points [
      let new-xcor mean [ xcor ] of my-points
      let new-ycor mean [ ycor ] of my-points
      if distancexy new-xcor new-ycor > movement-threshold [
        set any-centroids-moved? true
      ]
      setxy new-xcor new-ycor
    ]
  ]
  update-plots
end

to-report closest-centroid
  report min-one-of centroids [ distance myself ]
end


;;;-----------------------
;;;-----------------------
;;; Setup
;;;-----------------------
;;;-----------------------

to setup
  set maximumDelay 1
  clear-all
  set cleared? true
  if seed?
      [random-seed seed]

  setup-patches
  setup-individuals
  set-default-shape centroids "circle"
  set-default-shape movingcents "x"

  reset-ticks
end

to setup-patches
  ask patches[
    let allButijZero n-values dimensions [random-float 5]
    set allButijZero replace-item i allButijZero (pxcor * 0.0512)
    set allButijZero replace-item j allButijZero (pycor * 0.0512)

    set height function allButijZero
    set fitnessP -1 * height
    ;print (list allButijZero height)
  ]
    ; Check http://www.colorbrewer.org for different palletes
  let maxheight ([height] of max-one-of patches [height])
  ask patches [
    ;set pcolor scale-color grey height 0 1
    ;set pcolor palette:scale-gradient  [[255 0 0] [0 0 255]] height 0 1
    ;call from command center> palette:scheme-dialog
    set height height / maxheight
    set fitnessP -1 * height
    set pcolor palette:scale-gradient palette:scheme-colors  "Sequential"  "Greys" 3 height 0 1
  ]

end

to setup-individuals
  set-default-shape individuals "circle"
  ; put some turtles on patch centers in the landscape
  ask n-of population patches [
    sprout-individuals 1 [

      set posX n-values dimensions [(random-float 10.24) - 5.12]

      set tickdelay (random maximumDelay) + 1
      set currentdelay tickdelay
      set size ( tickdelay + 2 ) /  2

      set color [ 255 0 0 150]
      set fitness function posX
      set coopfitness fitness
      set nselections 0


      set xcor item i posX * 19.53125
      set ycor item j posX * 19.53125

      ;pen-down
    ]
  ]

  let maxfitness [fitness] of max-one-of individuals [fitness]

  ask individuals[
    set fitness fitness / maxfitness
    set coopfitness fitness
    set color palette:scale-gradient palette:scheme-colors   "Sequential"  "Reds" 3 fitness 1 0
  ]


  ;; Estimate distances

  if whichDistance = "nearestneighbor"
  [
    ask individuals[ set nn NearestNeighborDistance self individuals]
    ask individuals[ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nn] of self]
  ]

  if whichDistance = "nearestneighborscaled"
  [
    ask individuals[ set nn NearestNeighborDistance self individuals]
    ask individuals[
      let fitnessDifference abs ([fitness] of self - [fitness] of individual [nn] of self)
        let eucliDistance EuclideanDistance [posX] of self [posX] of individual [nn] of self
        ;show fitnessDifference
        ;show eucliDistance
        set chosendistance eucliDistance / (fitnessDifference * 1.0)
      ]
  ]


  if whichDistance = "bestneighbor"
  [
    ask individuals[ set nb BestNeighborDistance self individuals]

    ask individuals[
      ifelse [who] of self = [nb] of self
      [ set chosendistance 0 ]
      [ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nb] of self ]
    ]
  ]
  if whichDistance = "bestneighborscaled"
  [
    ask individuals[ set nb BestNeighborDistance self individuals]

    let minfitness min [fitness] of individuals

    ask individuals[
      ifelse [who] of self = [nb] of self
      [ set chosendistance 0 ]
      [ ;let fitnessDifference [fitness] of self - [fitness] of individual [nb] of self
        let fitnessDifference [fitness] of self - minfitness
        let eucliDistance EuclideanDistance [posX] of self [posX] of individual [nb] of self
        ;show fitnessDifference
        ;show eucliDistance
        set chosendistance eucliDistance / (fitnessDifference * 1.0)
      ]
    ]
  ]
  if whichDistance = "Mehalanobis"
  [
    let tabla stats:newtable
    ask individuals[ stats:add tabla [posx] of self  ]
    ask individuals[ set chosendistance MehalanobisDistance tabla [posx] of self ]
  ]
  if whichDistance = "globalbest"
  [
    ask individuals[ set nb GlobalBestDistance self individuals]
    ask individuals[
      ifelse [posX] of self = [posX] of individual [nb] of self
      [ set chosendistance 0 ]
      [ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nb] of self ]
    ]
  ]

end


to replot
  setup-patches
end


;;;-----------------------
;;;-----------------------
;;; Go
;;;-----------------------
;;;-----------------------

to go
  ; The following line of code looks a bit odd, so we'll explain it.
  ; if we simply wrote "LET OLD-GENERATION TURTLES",
  ; then OLD-GENERATION would mean the set of all turtles, and when
  ; new solutions were created, they would be added to the breed, and
  ; OLD-GENERATION would also grow.  Since we don't want it to grow,
  ; we instead write "TURTLES WITH [TRUE]", which makes OLD-GENERATION
  ; an agentset, which doesn't get updated when new solutions are created.
  set old-generation turtles with [true]
  set old-generation-list [self] of old-generation



  if (selection = "tournament")
      [ CompetitiveEvolution ]

  if (selection = "cooperative")
      [ CooperativeEvolution ]

  if (selection = "MOlove")
      [ MOlove ]

  if (selection = "MOrandom")
      [ MOrandom ]

  if (selection = "cooperative random rule")
      [ CooperativeRandomEvolution ]

  if (selection = "linear-ranking")
      [ LinearRankingEvolution ]

  if (selection = "distance")
      [ DistanceEvolution ]

  ifelse any? turtles with [fitness < [fitness] of max-one-of individuals [fitness]]
  [let a 0]
  [stop]

  ;; Estimate mehalanobis distance
  ;let tabla stats:newtable
  ;ask individuals[ stats:add tabla [posx] of self  ]
  ;ask individuals[ set mehalanobis MehalanobisDistance tabla [posx] of self ]
  if whichDistance = "nearestneighbor"
  [
    ask individuals[ set nn NearestNeighborDistance self individuals]
    ask individuals[ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nn] of self]
  ]

  if whichDistance = "nearestneighborscaled"
  [
    ask individuals[ set nn NearestNeighborDistance self individuals]
    ask individuals[
      let fitnessDifference abs ([fitness] of self - [fitness] of individual [nn] of self)
        let eucliDistance EuclideanDistance [posX] of self [posX] of individual [nn] of self
        ;show fitnessDifference
        ;show eucliDistance
        set chosendistance eucliDistance / (fitnessDifference * 1.0)
      ]
  ]


  if whichDistance = "bestneighbor"
  [
    ask individuals[ set nb BestNeighborDistance self individuals]

    ask individuals[
      ifelse [who] of self = [nb] of self
      [ set chosendistance 0 ]
      [ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nb] of self ]
    ]
  ]
  if whichDistance = "bestneighborscaled"
  [
    ask individuals[ set nb BestNeighborDistance self individuals]

    ask individuals[
      ifelse [who] of self = [nb] of self
      [ set chosendistance 0 ]
      [ let fitnessDifference [fitness] of self - [fitness] of individual [nb] of self
        let eucliDistance EuclideanDistance [posX] of self [posX] of individual [nb] of self
        set chosendistance eucliDistance / (fitnessDifference * 1.0)
      ]
    ]
  ]
  if whichDistance = "Mehalanobis"
  [
    let tabla stats:newtable
    ask individuals[ stats:add tabla [posx] of self  ]
    ask individuals[ set chosendistance MehalanobisDistance tabla [posx] of self ]
  ]
  if whichDistance = "globalbest"
  [
    ask individuals[ set nb GlobalBestDistance self individuals]
    ask individuals[
      ifelse [posX] of self = [posX] of individual [nb] of self
      [ set chosendistance 0 ]
      [ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nb] of self ]
    ]
  ]


  set old-generation turtle-set old-generation-list
  ask old-generation [ die ]

  tick
end


;;;---------------------------------
;;;CooperativeEvolution procedure
;;;---------------------------------
to CooperativeEvolution

    ask individuals[

    set currentdelay (currentdelay - 1)

    ifelse ( currentdelay = 0)[
      let association (turtle-set n-of (tournamentSize) old-generation)
      let orderedAssociation sort-on [(coopfitness)] association
      ;let orderedset (turtle-set orderedAssociation)



      let winner item 0 orderedAssociation
      let second item 1 orderedAssociation
      let third 0
      ifelse (length orderedAssociation < 3)
      [set third second]
      [set third item 2 orderedAssociation]

      let newCoop (([coopfitness] of second + [coopfitness] of third) / 2.0)
      ask winner [set coopfitness newCoop]
      ;let newCoop (([fitness] of second + [fitness] of third) / 2.0)
      ;let scaleCoop (((newCoop - minCoop) * 9.9999999) / (maxCoop - minCoop))

      ask self [ hatch 1 [
        set fitness [fitness] of winner
        ;set coopfitness ((newCoop + fitness) / 2.0)
        ;set coopfitness newCoop
        set coopfitness fitness
        set color [color] of winner
        setxy [xcor] of winner [ycor] of winner
        set posX [posX] of winner
        set currentdelay tickdelay
        ;set coopfitness scaleCoop
      ] ]
      set currentdelay tickdelay
    ]
    [set old-generation-list remove self old-generation-list ] ; If there is no selection, we stay in the population


    ;show [fitness] of self
  ]

end


;;;---------------------------------
;;;CooperativeRandomEvolution procedure
;;;---------------------------------
to CooperativeRandomEvolution

    ask individuals[

    set currentdelay (currentdelay - 1)

    ifelse ( currentdelay = 0)[
      let association (turtle-set n-of (tournamentSize) old-generation)
      let orderedAssociation sort-on [(coopfitness)] association

      ifelse random-float 1 < randomness
      [
        let winner one-of old-generation
        ask self [ hatch 1 [
          set fitness [fitness] of winner
          ;set coopfitness ((newCoop + fitness) / 2.0)
          ;set coopfitness newCoop
          set coopfitness fitness
          set color [color] of winner
          setxy [xcor] of winner [ycor] of winner
          set posX [posX] of winner
          set currentdelay tickdelay
        ;set coopfitness scaleCoop
        ] ]
      ]
      [
        let winner item 0 orderedAssociation
        let second item 1 orderedAssociation
        let third 0
        ifelse (length orderedAssociation < 3)
        [set third second]
        [set third item 2 orderedAssociation]

        let newCoop (([coopfitness] of second + [coopfitness] of third) / 2.0)
        ask winner [set coopfitness newCoop]
        ;let newCoop (([fitness] of second + [fitness] of third) / 2.0)
        ;let scaleCoop (((newCoop - minCoop) * 9.9999999) / (maxCoop - minCoop))

        ask self [ hatch 1 [
          set fitness [fitness] of winner
          ;set coopfitness ((newCoop + fitness) / 2.0)
          ;set coopfitness newCoop
          set coopfitness fitness
          set color [color] of winner
          setxy [xcor] of winner [ycor] of winner
          set posX [posX] of winner
          set currentdelay tickdelay
        ;set coopfitness scaleCoop
        ] ]
      ]
      set currentdelay tickdelay
    ]
    [set old-generation-list remove self old-generation-list ] ; If there is no selection, we stay in the population


    ;show [fitness] of self
  ]

end



;;;---------------------------------
;;;CompetitiveEvolution procedure
;;;---------------------------------
to CompetitiveEvolution


    ask individuals[

    set currentdelay (currentdelay - 1)

    ifelse ( currentdelay = 0)[
      let winner item 0 sort-on [(fitness)] (turtle-set n-of (tournamentSize) old-generation)

      ask self [ hatch 1 [
        set fitness [fitness] of winner
        set coopfitness fitness
        set color [color] of winner
        setxy [xcor] of winner [ycor] of winner
        set posX [posX] of winner
        set currentdelay tickdelay
        ;set coopfitness scaleCoop
      ] ]
      set currentdelay tickdelay
    ]
    [set old-generation-list remove self old-generation-list  ] ; If there is no selection, we stay in the population

    ;show [fitness] of self
  ]

end

;;;---------------------------------
;;;DistanceEvolution procedure
;;;---------------------------------
to DistanceEvolution

     ask individuals[

    set currentdelay (currentdelay - 1)

    ifelse ( currentdelay = 0)[
      ifelse random-float 1 < randomness
      [
         let selected (turtle-set n-of (tournamentSize) old-generation)
         let tbl stats:newtable
         ask selected[
            stats:add tbl [posx] of self
         ]


         ask selected[
         set chosendistance MehalanobisDistance tbl [posx] of self
         ]

         let winner item 0 sort-on [(- chosendistance)] selected
         ask self [ hatch 1 [
        set fitness [fitness] of winner
        set coopfitness fitness
        set color [color] of winner
        setxy [xcor] of winner [ycor] of winner
        set posX [posX] of winner
        set currentdelay tickdelay
        ;set coopfitness scaleCoop
      ] ]
      ]
      [
        let association (turtle-set n-of (tournamentSize) old-generation)
        let orderedAssociation sort-on [(coopfitness)] association
        let winner item 0 orderedAssociation
        let second item 1 orderedAssociation
        let third 0
        ifelse (length orderedAssociation < 3)
        [set third second]
        [set third item 2 orderedAssociation]

        let newCoop (([coopfitness] of second + [coopfitness] of third) / 2.0)
        ask winner [set coopfitness newCoop]

        ask self [ hatch 1 [
        set fitness [fitness] of winner
        set coopfitness fitness
        set color [color] of winner
        setxy [xcor] of winner [ycor] of winner
        set posX [posX] of winner
        set currentdelay tickdelay
        ;set coopfitness scaleCoop
      ] ]
      ]

      set currentdelay tickdelay
    ]
    [set old-generation-list remove self old-generation-list  ] ; If there is no selection, we stay in the population


    ;show [fitness] of self
  ]

end

;;;---------------------------------
;;;CompetitiveEvolution procedure
;;;---------------------------------
to LinearRankingEvolution


    ask individuals[

    set currentdelay (currentdelay - 1)

    let ranked-population sort-on [(fitness)] old-generation

    ifelse ( currentdelay = 0)[

      let popsize count old-generation

      let index popsize * (bias - sqrt ((bias * bias) - 4 * (bias - 1) * random-float 1)) / 2.0 / (bias - 1)

      let winner item index ranked-population

      ask self [ hatch 1 [
        set fitness [fitness] of winner
        set coopfitness fitness
        set color [color] of winner
        setxy [xcor] of winner [ycor] of winner
        set currentdelay tickdelay
        set posX [posX] of winner
        ;set coopfitness scaleCoop
      ] ]
      set currentdelay tickdelay
    ]
    [set old-generation-list remove self old-generation-list  ] ; If there is no selection, we stay in the population

    ;show [fitness] of self
  ]

end

;;;---------------------------------
;;;MOrandom procedure
;;;---------------------------------
to MOrandom

    ask individuals[

      let association (turtle-set n-of (tournamentSize) old-generation)
      let orderedAssociation sort-on [(coopfitness)] association
      ;let orderedset (turtle-set orderedAssociation)

          if whichDistance = "nearestneighbor"
          [
            ask association[ set nn NearestNeighborDistance self association]
            ask association[ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nn] of self]
          ]
          if whichDistance = "bestneighbor"
          [
            ask association[ set nb BestNeighborDistance self association]

            ask association[
              ifelse [who] of self = [nb] of self
              [ set chosendistance 0 ]
              [ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nb] of self ]
            ]
          ]
          if whichDistance = "globalbest"
          [
            ask association[ set nb GlobalBestDistance self association]
            ask association[
              ifelse [posX] of self = [posX] of individual [nb] of self
              [ set chosendistance 0 ]
              [ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nb] of self ]
            ]
          ]

      let front sort-on [(coopfitness)] ParetoFront association

      let winner one-of front

      ask self [ hatch 1 [
        set fitness [fitness] of winner
        ;set coopfitness ((newCoop + fitness) / 2.0)
        ;set coopfitness newCoop
        set coopfitness fitness
        set color [color] of winner
        setxy [xcor] of winner [ycor] of winner
        set posX [posX] of winner
       ;set coopfitness scaleCoop
      ] ]

    ;show [fitness] of self
  ]



end

;;;---------------------------------
;;;MOlove procedure
;;;---------------------------------
to MOlove

    ask individuals[

      let association (turtle-set n-of (tournamentSize) old-generation)
      let orderedAssociation sort-on [(coopfitness)] association
      ;let orderedset (turtle-set orderedAssociation)

          if whichDistance = "nearestneighbor"
          [
            ask association[ set nn NearestNeighborDistance self association]
            ask association[ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nn] of self]
          ]
          if whichDistance = "bestneighbor"
          [
            ask association[ set nb BestNeighborDistance self association]

            ask association[
              ifelse [who] of self = [nb] of self
              [ set chosendistance 0 ]
              [ set chosendistance EuclideanDistance [posX] of self [posX] of individual [nb] of self ]
            ]
          ]

      let front sort-on [(coopfitness)] ParetoFront association

      ;show association
      ;show front

      let winner item 0 front
      let second 0
      ifelse (length front < 2)
      [set second winner]
      [set second item 1 front]

      let third 0
      ifelse (length front < 3)
      [set third second]
      [set third item 2 front]

      let newCoop (([coopfitness] of second + [coopfitness] of third) / 2.0)
      ask winner [set coopfitness newCoop]
      ;let newCoop (([fitness] of second + [fitness] of third) / 2.0)
      ;let scaleCoop (((newCoop - minCoop) * 9.9999999) / (maxCoop - minCoop))

      ask self [ hatch 1 [
        set fitness [fitness] of winner
        ;set coopfitness ((newCoop + fitness) / 2.0)
        set coopfitness newCoop
        ;set coopfitness fitness
        set color [color] of winner
        setxy [xcor] of winner [ycor] of winner
        set posX [posX] of winner
       ;set coopfitness scaleCoop
      ] ]

    ;show [fitness] of self
  ]

end



;;;
;;; Plotting functions
;;;
to ScatterPlot
   clear-plot
   set-current-plot-pen "D-pen"

   let list-1 list 0 0
   let list-2 list 0 0

   ask individuals[
     set list-1 lput [fitness] of self list-1
     set list-2 lput [chosendistance] of self list-2
     set size 2
   ]

   set list-1 butfirst list-1
   set list-2 butfirst list-2
   set list-1 butfirst list-1
   set list-2 butfirst list-2


   (foreach list-1 list-2 [plotxy ?1 ?2])
end



to ParetoPlot
   set-current-plot-pen "P-pen"
   let front ParetoFront individuals
   let list-1 list 0 0
   let list-2 list 0 0

   ask front[
     set list-1 lput [fitness] of self list-1
     set list-2 lput [chosendistance] of self list-2
     set size 2
   ]

   set list-1 butfirst list-1
   set list-2 butfirst list-2
   set list-1 butfirst list-1
   set list-2 butfirst list-2
   (foreach list-1 list-2 [plotxy ?1 ?2])
end



to TestParetoPlot

  let selected (turtle-set n-of (tournamentSize) individuals)
  let tbl stats:newtable
  ask selected[
    stats:add tbl [posx] of self
  ]
  ask selected[
     ;set mehalanobis MehalanobisDistance tbl [posx] of self
  ]

  ask selected[
;       set-current-plot-pen "red"
;       plotxy [fitness] of self [mehalanobis] of self
  ]


   set-current-plot-pen "red"
   let front ParetoFront selected
   let list-1 list 0 0
   let list-2 list 0 0

   ask front[
     set list-1 lput [fitness] of self list-1
     set list-2 lput [chosendistance] of self list-2
     ;set size 8
   ]

   set list-1 butfirst list-1
   set list-2 butfirst list-2
   set list-1 butfirst list-1
   set list-2 butfirst list-2
   (foreach list-1 list-2 [plotxy ?1 ?2])
end

;;;-----------------------
;;;-----------------------
;;; Pareto front Fitness/Distance
;;;-----------------------
;;;-----------------------
to-report ParetoFront [AgentSet]

  let A sort-on [fitness] AgentSet
  let front list 0 item 0 A

  let iindex 0
  let jindex 1
  while [jindex < length A]
  [
    let currentdistance [chosendistance] of item iindex A
    let nextdistance [chosendistance] of item jindex A
    let currentfitness [fitness] of item iindex A
    let nextfitness [fitness] of item jindex A


    if (currentdistance <= nextdistance or currentfitness = nextfitness )
    [
          set front lput item jindex A front
          set iindex jindex
    ]

    set jindex jindex + 1
  ]

  ;show front

  report turtle-set butfirst front
end

;;;-----------------------
;;;-----------------------
;;; Mehalanobis distance
;;;-----------------------
;;;-----------------------

to-report MehalanobisDistance [DistributionTable vectorX]
  let S      matrix:from-row-list stats:covariance DistributionTable
  let S-1    matrix:inverse S

  let X      matrix:from-row-list (list vectorX)
  let MU     matrix:from-row-list (list stats:means DistributionTable)

  let X-MU   X matrix:- MU
  let X-MU-T matrix:transpose X-MU

  ;print "--------"
  ;show X-MU-T
  ;print "--------"
  ;show X-MU
  ;print "--------"
  ;show S-1
  ;print "--------"

  let mult X-MU matrix:* S-1 matrix:* X-MU-T

  report sqrt matrix:get mult 0 0
end



;;;-----------------------
;;;-----------------------
;;; Best neighbor distance
;;;-----------------------
;;;-----------------------

to-report BestNeighborDistance [Agent AgentSet]

  let betterIndividus individuals with [fitness < [fitness] of Agent]
  ifelse betterIndividus = nobody
  [
    report [who] of Agent
  ]
  [
    let listInd sort-by < [who] of betterIndividus
    let distancelist list 999999 999999 ; a strange way of initializing the list
    let individulist list [who] of Agent [who] of Agent ; a strange way of initializing the list

    foreach listInd
    [
      let newdistance EuclideanDistance [posX] of Agent [posX] of individual ?
      set distancelist lput newdistance distancelist
      set individulist lput ? individulist
    ]

    let nbdistance min distancelist
    let nbposition position nbdistance distancelist

    report item nbposition individulist
  ]

end


;;;-----------------------
;;;-----------------------
;;; Nearest neighbor distance
;;;-----------------------
;;;-----------------------

to-report NearestNeighborDistance [Agent AgentSet]

  let listInd sort-by < [who] of AgentSet
  let distancelist list 999999 999999 ; a strange way of initializing the list
  let individulist list [who] of Agent [who] of Agent ; a strange way of initializing the list

  foreach listInd
  [
    let newdistance EuclideanDistance [posX] of Agent [posX] of individual ?
    set distancelist lput newdistance distancelist
    set individulist lput ? individulist
  ]

  let nndistance min distancelist
  let nnposition position nndistance distancelist

  report item nnposition individulist
end

;;;-----------------------
;;;-----------------------
;;; Global best distance
;;;-----------------------
;;;-----------------------

to-report GlobalBestDistance [Agent AgentSet]

  let listInd sort-on [fitness] AgentSet
  report [who] of first listInd

end


;;;-----------------------
;;;-----------------------
;;; Euclidean distance
;;;-----------------------
;;;-----------------------

to-report EuclideanDistance [posA posB]

  let diff (map - posA posB)
  let len length diff
  let two n-values len [2]
  let square (map ^ diff two)
  let sumatory reduce + square
  let result sqrt sumatory
  ifelse result > 0
     [report result]
     [report 999999] ; Used as infinity
end


;;;-----------------------
;;;-----------------------
;;; Functions
;;;-----------------------
;;;-----------------------

to-report function [X]

  if landscape = "rastrigin"
  [report rastrigin X]

  if landscape = "sphere"
  [report sphere X]

  if landscape = "juanlugin"
  [report juanlugin X]

  if landscape = "rotatedjuanlugin"
  [report rotatedjuanlugin X]
end


to-report rastrigin [X]

  let sumi 10 * dimensions

  foreach X
  [
    ;;uncomment next line for NetLogo 6.0
    ;;xi -> set sumi (sumi + ( xi ^ 2 - 10 * cos (2 * pi * xi * 57.2958) ))
    ;;uncomment next line for NetLogo 5.3
    set sumi (sumi + ( ? ^ 2 - 10 * cos (2 * pi * ? * 57.2958) ))
  ]

  report sumi

end


to-report juanlugin [X]

  let sumi 10 * dimensions

  foreach X
  [
    ;;uncomment next line for NetLogo 6.0
    ;xi -> set sumi (sumi - ( 10 * cos (2 * pi * xi * 57.2958) ))
    ;;uncomment next line for NetLogo 5.3
    set sumi (sumi - ( 10 * cos (2 * pi * ? * 57.2958) ))
  ]

  report sumi

end


to-report rotatedjuanlugin [Z]

  let sumi 10 * dimensions

  let x item 0 Z
  let y item 1 Z

  let xp x * cos angle - y * sin angle
  let yp x * sin angle - y * cos angle

  set x xp / 2.0
  set y yp / 2.0


  let COOR list x y



  foreach COOR
  [
    ;;uncomment next line for NetLogo 6.0
    ;xi -> set sumi (sumi - ( 10 * cos (2 * pi * xi * 57.2958) ))
    ;;uncomment next line for NetLogo 5.3
    set sumi (sumi - ( 10 * cos (2 * pi * ? * 57.2958) ))
  ]

  report sumi

end


to-report sphere [X]

  let sumi 0

  foreach X
  [
    ;;uncomment next line for NetLogo 6.0
    ;;xi -> set sumi (sumi + ( xi ^ 2 ))
    ;;uncomment next line for NetLogo 5.3
    set sumi (sumi + ( ? ^ 2 ))
  ]

  report sumi

end
@#$#@#$#@
GRAPHICS-WINDOW
304
10
741
468
100
100
2.1244
1
10
1
1
1
0
0
0
1
-100
100
-100
100
0
0
1
ticks
30.0

SLIDER
14
199
186
232
dimensions
dimensions
2
100
2
1
1
NIL
HORIZONTAL

SWITCH
6
91
109
124
seed?
seed?
1
1
-1000

INPUTBOX
5
31
108
91
seed
67285
1
0
Number

SLIDER
16
287
188
320
population
population
0
2000
2000
1
1
NIL
HORIZONTAL

BUTTON
478
470
534
504
NIL
replot
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
304
470
476
503
i
i
0
dimensions - 1
1
1
1
NIL
HORIZONTAL

SLIDER
272
293
305
443
j
j
0
dimensions - 1
0
1
1
NIL
VERTICAL

TEXTBOX
140
16
290
106
Plot of (x_i , x_j) values. The rest of x_h is assumed to be 0, with h not being i or j.\n\nWARNING i!=j
12
0.0
1

BUTTON
214
110
288
143
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

CHOOSER
14
153
183
198
landscape
landscape
"rastrigin" "sphere" "juanlugin" "rotatedjuanlugin"
0

BUTTON
221
145
284
178
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
222
181
286
214
step
repeat 1 [go]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
42
134
192
152
LANDSCAPE
12
0.0
1

TEXTBOX
51
265
201
283
ALGORITHM
12
0.0
1

CHOOSER
17
322
243
367
selection
selection
"tournament" "cooperative" "MOlove" "MOrandom" "linear-ranking" "cooperative random rule" "distance"
1

TEXTBOX
42
486
192
504
K-means
12
0.0
1

SLIDER
17
368
189
401
tournamentSize
tournamentSize
2
200
8
1
1
NIL
HORIZONTAL

SLIDER
18
402
190
435
bias
bias
1
2
1.96
0.01
1
NIL
HORIZONTAL

SLIDER
11
509
273
542
num-centroids
num-centroids
1
121
121
1
1
NIL
HORIZONTAL

PLOT
753
10
1072
160
FRD
NIL
NIL
0.0
1.0
0.0
10.0
true
false
"" ""
PENS
"default" 0.01 1 -16777216 true "" "histogram [fitness] of individuals"

PLOT
754
162
1070
328
Diversity
NIL
NIL
0.0
10.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "plot (length remove-duplicates [posx] of individuals) / population"

MONITOR
305
502
365
547
different ind.
length remove-duplicates [posx] of individuals
17
1
11

SLIDER
18
436
190
469
randomness
randomness
0
1
0.87
0.01
1
NIL
HORIZONTAL

BUTTON
223
217
286
250
test
TestParetoPlot
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
756
333
1072
493
div vs. fitness
NIL
NIL
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"D-pen" 1.0 2 -16777216 true "" "ScatterPlot"
"P-pen" 10.0 2 -13791810 true "" "ParetoPlot"
"red" 1.0 2 -2674135 true "" ""

CHOOSER
534
473
751
518
WhichDistance
WhichDistance
"Mehalanobis" "nearestneighbor" "nearestneighborscaled" "bestneighbor" "bestneighborscaled" "globalbest"
5

BUTTON
12
543
116
576
NIL
reset-centroids
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
126
543
199
576
k-means
repeat 100 [k-means]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
201
543
272
576
clean
ask centroids [die]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
223
254
283
287
clean
ask individuals [die]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
13
233
185
266
angle
angle
0
180
18
1
1
NIL
HORIZONTAL

BUTTON
15
606
180
639
NIL
reset-movingcents
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
49
587
199
605
Shift-mean clustering
12
0.0
1

BUTTON
196
606
266
639
move
repeat 100 [\nmove-movingcents]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
275
606
338
639
kill
kill-movingcents-in-radius
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
513
586
587
631
centroids
length [xcor] of movingcents + length [xcor] of centroids
17
1
11

BUTTON
347
606
418
639
clean
;ask movingcents[die]\nclear-drawing
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
646
598
942
631
Uphill
clear-drawing\nrepeat 100[\n   ask movingcents [\n   uphill fitnessP\n]\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

TEXTBOX
778
578
928
596
Uphill
12
0.0
1

@#$#@#$#@
## WHAT IS IT?

(a general understanding of what the model is trying to show or explain)

## HOW IT WORKS

(what rules the agents use to create the overall behavior of the model)

## HOW TO USE IT

(how to use the model, including a description of each of the items in the Interface tab)

## THINGS TO NOTICE

(suggested things for the user to notice while running the model)

## THINGS TO TRY

(suggested things for the user to try to do (move sliders, switches, etc.) with the model)

## EXTENDING THE MODEL

(suggested things to add or change in the Code tab to make the model more complicated, detailed, accurate, etc.)

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.3.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
